// zakladni ukazka kanalu
package main

import (
	"fmt"
	"math/rand/v2"
	"time"
)

func channelExample() {
	fmt.Println("\n---CHANNEL 1---")
	ch := make(chan int) //vytvorime kanal pro int hodnoty
	go func() {
		time.Sleep(2 * time.Second)
		ch <- 42 // posleme hodnotu do kanalu
	}()
	value := <-ch // cekame na hodnotu z kanalu
	fmt.Println(value)
}

func selectExample() {
	fmt.Println("\n---CHANNEL SELECT---")
	ch1 := make(chan int)
	ch2 := make(chan int)
	go func() {
		time.Sleep(2 * time.Second)
		if rand.Int()%2 == 0 {
			ch1 <- 1
		} else {
			ch2 <- 2
		}
	}()

	for {
		select {
		case msg := <-ch1:
			fmt.Println("Received from ch1:", msg)
			return
		case msg := <-ch2:
			fmt.Println("Received from ch2:", msg)
			return
		default:
			fmt.Println("No messages received")
			time.Sleep(300 * time.Millisecond)
		}
	}
}

func chanWorker(ch chan<- int) { //kanal jen pro zapis
	for i := 1; i <= 5; i++ {
		ch <- i
		//x := <-ch  //zkusme cist z kanalu
		fmt.Println("Sent:", i)
		time.Sleep(200 * time.Millisecond)
	}
	close(ch) // dalsi data nebudou, zavirame kanal
}

func complexChannelExample() {
	fmt.Println("\n---CHANNEL 3---")
	ch := make(chan int, 3) // bufferovany kanal (kapacita bufferu 3)
	go chanWorker(ch)
	for val := range ch { // dokud kanal neni zavreny prijimame (range iteruje hodnoty z kanalu, dokud neni uzavren)
		fmt.Println("Received:", val)
	}

	fmt.Println("Done receiving")
}
